#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2000-2006  Donald N. Allingham
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#



"""
Contains the interface to allow to write place data: longitude latitude. 
One backend for writing is supported: the KML file format used by google earth

info: bm at cage dot UGent dot be
"""

#-------------------------------------------------------------------------
#
# load standard python libraries
#
#-------------------------------------------------------------------------
import time
import os
import codecs
from gettext import gettext as _
import Utils

#-------------------------------------------------------------------------
#
# load GRAMPS libraries
#
#-------------------------------------------------------------------------
import RelLib

#------------------------------------------------------------------------
#
# Set up logging
#
#------------------------------------------------------------------------
import logging
log = logging.getLogger(".WriteGreographicData")


#-------------------------------------------------------------------------
#
#   backend: KML writer for the actual writing of geographic data
#
#-------------------------------------------------------------------------
class KmlWriter:
    """
    Writes output to a KML file.
    """
    def __init__(self, title, placemarks={}, placestyles={}):
        """
        Initializes, but does not write, a KML file.
           title : title of the KML group
           placestyle: Different styles to show data in kml. A dictionary with 
                       key stylename, data style
           placemarks: A dictionary with the places that must be shown, 
                    the index is the placehandle, the data with the index is
                    [placename, lat, lon, events], where events is a list with
                    every element a list [(personname/familyname, 
                        personname/familyname_short, type = F,M,U,Fam), 
                        eventname, eventdescription,
                        eventRole, eventDate, RegularDateTrueFalse], 
                    where RegularDateTrueFalse is True if eventDate is a 
                    regular date. The type is F = Female, M = Male, U = unknown
                    for personal events, and Fam for family events
        """
        self.title = title
        self.styles = placestyles
        self.places = placemarks
        #debug
        print 'KML init ' , placemarks
        
    def write(self,filename):
        """
        Write the data to the specified file.
        """
        base = os.path.dirname(filename)
        if os.path.isdir(base):
            if not os.access(base,os.W_OK) or not os.access(base,os.R_OK):
                ErrorDialog(_('Failure writing %s') % filename,
                            _("The file cannot be saved because you do "
                              "not have permission to write to the directory. "
                              "Please make sure you have write access to the "
                              "directory and try again."))
                return 0
            
        if os.path.exists(filename):
            if not os.access(filename,os.W_OK):
                ErrorDialog(_('Failure writing %s') % filename,
                            _("The file cannot be saved because you do "
                              "not have permission to write to the file. "
                              "Please make sure you have write access to the "
                              "file and try again."))
                return 0
        
        self.fileroot = os.path.dirname(filename)

        try:
            g = open(filename,"w")
        except IOError,msg:
            ErrorDialog(_('Failure writing %s') % filename,msg)
            return 0
        self.g = codecs.getwriter("utf8")(g)

        self.write_kml_data()
        g.close()
        return 1

    def write_handle(self,handle):
        """
        Write the data to the specified file handle.
        """
        g = handle

        self.g = codecs.getwriter("utf8")(g)

        self.write_kml_data()
        g.close()
        return 1
            
    def write_kml_data(self):

        date = time.localtime(time.time())

        place_len = len(self.places)

        self.g.write('<?xml version="1.0" encoding="UTF-8"?>\n')
        self.g.write('<kml xmlns="http://earth.google.com/kml/2.1">\n')
        # some info
        self.g.write('<!-- created date="%04d-%02d-%02d\" -->\n'%
                     (date[0],date[1],date[2]) )

        # start the document
        self.g.write("<Document>\n")

        # set the name of the document
        self.g.write("<name>%s</name> \n" %(self.title))
        # set the default styles used
        self.g.write("""
          <Style id="grampsM"> 
            <BalloonStyle>
              <!-- a background color for the balloon -->
              <color>00ffffff</color>
              <!-- styling of the balloon text -->
              <text><![CDATA[
              <b><font color="#CC0000" size="+3">$[name]</font></b>
              <br/><br/>
              $[description]
              <br/>
              ]]></text>
            </BalloonStyle>
            <IconStyle id="grampsIcon">
              <color>ffffffff</color>
              <colorMode>normal</colorMode>
              <scale>1</scale>
              <Icon>%s</Icon> 
              <hotSpot x="0.5"  y="0.5" 
                  xunits="fraction" yunits="fraction"/>    <!-- kml:vec2Type -->
            </IconStyle>
          </Style>
        """ % (os.path.join(const.image_dir, "KML","eventM.png")) )
        self.g.write("""
          <Style id="grampsF"> 
            <BalloonStyle>
              <!-- a background color for the balloon -->
              <color>00ffffff</color>
              <!-- styling of the balloon text -->
              <text><![CDATA[
              <b><font color="#CC0000" size="+3">$[name]</font></b>
              <br/><br/>
              $[description]
              <br/>
              ]]></text>
            </BalloonStyle>
            <IconStyle id="grampsIcon">
              <color>ffffffff</color>
              <colorMode>normal</colorMode>
              <scale>1</scale>
              <Icon>%s</Icon> 
              <hotSpot x="0.5"  y="0.5" 
                  xunits="fraction" yunits="fraction"/>    <!-- kml:vec2Type -->
            </IconStyle>
          </Style>
        """ % (os.path.join(const.image_dir, "KML","eventF.png")) )
        self.g.write("""
          <Style id="grampsU"> 
            <BalloonStyle>
              <!-- a background color for the balloon -->
              <color>00ffffff</color>
              <!-- styling of the balloon text -->
              <text><![CDATA[
              <b><font color="#CC0000" size="+3">$[name]</font></b>
              <br/><br/>
              $[description]
              <br/>
              ]]></text>
            </BalloonStyle>
            <IconStyle id="grampsIcon">
              <color>ffffffff</color>
              <colorMode>normal</colorMode>
              <scale>1</scale>
              <Icon>%s</Icon> 
              <hotSpot x="0.5"  y="0.5" 
                  xunits="fraction" yunits="fraction"/>    <!-- kml:vec2Type -->
            </IconStyle>
          </Style>
        """ % (os.path.join(const.image_dir, "KML","eventU.png")) )
        self.g.write("""
          <Style id="grampsFam"> 
            <BalloonStyle>
              <!-- a background color for the balloon -->
              <color>00ffffff</color>
              <!-- styling of the balloon text -->
              <text><![CDATA[
              <b><font color="#CC0000" size="+3">$[name]</font></b>
              <br/><br/>
              $[description]
              <br/>
              <br/>
              <!-- insert the to/from hyperlinks -->
              $[geDirections]
              ]]></text>
            </BalloonStyle>
            <IconStyle id="grampsIcon">
              <color>ffffffff</color>
              <colorMode>normal</colorMode>
              <scale>1</scale>
              <Icon>%s</Icon> 
              <hotSpot x="0.5"  y="0.5" 
                  xunits="fraction" yunits="fraction"/>    <!-- kml:vec2Type -->
            </IconStyle>
          </Style>
        """ % (os.path.join(const.image_dir, "KML","eventFam.png")) )
        self.g.write("""
          <Style id="grampsFammar"> 
            <BalloonStyle>
              <!-- a background color for the balloon -->
              <color>00ffffff</color>
              <!-- styling of the balloon text -->
              <text><![CDATA[
              <b><font color="#CC0000" size="+3">$[name]</font></b>
              <br/><br/>
              $[description]
              <br/>
              <br/>
              <!-- insert the to/from hyperlinks -->
              $[geDirections]
              ]]></text>
            </BalloonStyle>
            <IconStyle id="grampsIcon">
              <color>ffffffff</color>
              <colorMode>normal</colorMode>
              <scale>1</scale>
              <Icon>%s</Icon> 
              <hotSpot x="0.5"  y="0.5" 
                  xunits="fraction" yunits="fraction"/>    <!-- kml:vec2Type -->
            </IconStyle>
          </Style>
        """ % (os.path.join(const.image_dir, "KML","eventFammar.png")) )
        #icons events female
        self.g.write("""
          <Style id="grampsFbapt"> 
            <BalloonStyle>
              <!-- a background color for the balloon -->
              <color>00ffffff</color>
              <!-- styling of the balloon text -->
              <text><![CDATA[
              <b><font color="#CC0000" size="+3">$[name]</font></b>
              <br/><br/>
              $[description]
              <br/>
              <br/>
              <!-- insert the to/from hyperlinks -->
              $[geDirections]
              ]]></text>
            </BalloonStyle>
            <IconStyle id="grampsIcon">
              <color>ffffffff</color>
              <colorMode>normal</colorMode>
              <scale>1</scale>
              <Icon>%s</Icon> 
              <hotSpot x="0.5"  y="0.5" 
                  xunits="fraction" yunits="fraction"/>    <!-- kml:vec2Type -->
            </IconStyle>
          </Style>
        """ % (os.path.join(const.image_dir, "KML","eventFbapt.png")) )
        self.g.write("""
          <Style id="grampsFbirth"> 
            <BalloonStyle>
              <!-- a background color for the balloon -->
              <color>00ffffff</color>
              <!-- styling of the balloon text -->
              <text><![CDATA[
              <b><font color="#CC0000" size="+3">$[name]</font></b>
              <br/><br/>
              $[description]
              <br/>
              <br/>
              <!-- insert the to/from hyperlinks -->
              $[geDirections]
              ]]></text>
            </BalloonStyle>
            <IconStyle id="grampsIcon">
              <color>ffffffff</color>
              <colorMode>normal</colorMode>
              <scale>1</scale>
              <Icon>%s</Icon> 
              <hotSpot x="0.5"  y="0.5" 
                  xunits="fraction" yunits="fraction"/>    <!-- kml:vec2Type -->
            </IconStyle>
          </Style>
        """ % (os.path.join(const.image_dir, "KML","eventFbirth.png")) )
        self.g.write("""
          <Style id="grampsFdeath"> 
            <BalloonStyle>
              <!-- a background color for the balloon -->
              <color>00ffffff</color>
              <!-- styling of the balloon text -->
              <text><![CDATA[
              <b><font color="#CC0000" size="+3">$[name]</font></b>
              <br/><br/>
              $[description]
              <br/>
              <br/>
              <!-- insert the to/from hyperlinks -->
              $[geDirections]
              ]]></text>
            </BalloonStyle>
            <IconStyle id="grampsIcon">
              <color>ffffffff</color>
              <colorMode>normal</colorMode>
              <scale>1</scale>
              <Icon>%s</Icon> 
              <hotSpot x="0.5"  y="0.5" 
                  xunits="fraction" yunits="fraction"/>    <!-- kml:vec2Type -->
            </IconStyle>
          </Style>
        """ % (os.path.join(const.image_dir, "KML","eventFdeath.png")) )
        self.g.write("""
          <Style id="grampsFburial"> 
            <BalloonStyle>
              <!-- a background color for the balloon -->
              <color>00ffffff</color>
              <!-- styling of the balloon text -->
              <text><![CDATA[
              <b><font color="#CC0000" size="+3">$[name]</font></b>
              <br/><br/>
              $[description]
              <br/>
              <br/>
              <!-- insert the to/from hyperlinks -->
              $[geDirections]
              ]]></text>
            </BalloonStyle>
            <IconStyle id="grampsIcon">
              <color>ffffffff</color>
              <colorMode>normal</colorMode>
              <scale>1</scale>
              <Icon>%s</Icon> 
              <hotSpot x="0.5"  y="0.5" 
                  xunits="fraction" yunits="fraction"/>    <!-- kml:vec2Type -->
            </IconStyle>
          </Style>
        """ % (os.path.join(const.image_dir, "KML","eventFburial.png")) )
        #icons male
        self.g.write("""
          <Style id="grampsMbapt"> 
            <BalloonStyle>
              <!-- a background color for the balloon -->
              <color>00ffffff</color>
              <!-- styling of the balloon text -->
              <text><![CDATA[
              <b><font color="#CC0000" size="+3">$[name]</font></b>
              <br/><br/>
              $[description]
              <br/>
              <br/>
              <!-- insert the to/from hyperlinks -->
              $[geDirections]
              ]]></text>
            </BalloonStyle>
            <IconStyle id="grampsIcon">
              <color>ffffffff</color>
              <colorMode>normal</colorMode>
              <scale>1</scale>
              <Icon>%s</Icon> 
              <hotSpot x="0.5"  y="0.5" 
                  xunits="fraction" yunits="fraction"/>    <!-- kml:vec2Type -->
            </IconStyle>
          </Style>
        """ % (os.path.join(const.image_dir, "KML","eventMbapt.png")) )
        self.g.write("""
          <Style id="grampsMbirth"> 
            <BalloonStyle>
              <!-- a background color for the balloon -->
              <color>00ffffff</color>
              <!-- styling of the balloon text -->
              <text><![CDATA[
              <b><font color="#CC0000" size="+3">$[name]</font></b>
              <br/><br/>
              $[description]
              <br/>
              <br/>
              <!-- insert the to/from hyperlinks -->
              $[geDirections]
              ]]></text>
            </BalloonStyle>
            <IconStyle id="grampsIcon">
              <color>ffffffff</color>
              <colorMode>normal</colorMode>
              <scale>1</scale>
              <Icon>%s</Icon> 
              <hotSpot x="0.5"  y="0.5" 
                  xunits="fraction" yunits="fraction"/>    <!-- kml:vec2Type -->
            </IconStyle>
          </Style>
        """ % (os.path.join(const.image_dir, "KML","eventMbirth.png")) )
        self.g.write("""
          <Style id="grampsMdeath"> 
            <BalloonStyle>
              <!-- a background color for the balloon -->
              <color>00ffffff</color>
              <!-- styling of the balloon text -->
              <text><![CDATA[
              <b><font color="#CC0000" size="+3">$[name]</font></b>
              <br/><br/>
              $[description]
              <br/>
              <br/>
              <!-- insert the to/from hyperlinks -->
              $[geDirections]
              ]]></text>
            </BalloonStyle>
            <IconStyle id="grampsIcon">
              <color>ffffffff</color>
              <colorMode>normal</colorMode>
              <scale>1</scale>
              <Icon>%s</Icon> 
              <hotSpot x="0.5"  y="0.5" 
                  xunits="fraction" yunits="fraction"/>    <!-- kml:vec2Type -->
            </IconStyle>
          </Style>
        """ % (os.path.join(const.image_dir, "KML","eventMdeath.png")) )
        self.g.write("""
          <Style id="grampsMburial"> 
            <BalloonStyle>
              <!-- a background color for the balloon -->
              <color>00ffffff</color>
              <!-- styling of the balloon text -->
              <text><![CDATA[
              <b><font color="#CC0000" size="+3">$[name]</font></b>
              <br/><br/>
              $[description]
              <br/>
              <br/>
              <!-- insert the to/from hyperlinks -->
              $[geDirections]
              ]]></text>
            </BalloonStyle>
            <IconStyle id="grampsIcon">
              <color>ffffffff</color>
              <colorMode>normal</colorMode>
              <scale>1</scale>
              <Icon>%s</Icon> 
              <hotSpot x="0.5"  y="0.5" 
                  xunits="fraction" yunits="fraction"/>    <!-- kml:vec2Type -->
            </IconStyle>
          </Style>
        """ % (os.path.join(const.image_dir, "KML","eventMburial.png")) )
        #icons unknown
        self.g.write("""
          <Style id="grampsUbapt"> 
            <BalloonStyle>
              <!-- a background color for the balloon -->
              <color>00ffffff</color>
              <!-- styling of the balloon text -->
              <text><![CDATA[
              <b><font color="#CC0000" size="+3">$[name]</font></b>
              <br/><br/>
              $[description]
              <br/>
              <br/>
              <!-- insert the to/from hyperlinks -->
              $[geDirections]
              ]]></text>
            </BalloonStyle>
            <IconStyle id="grampsIcon">
              <color>ffffffff</color>
              <colorMode>normal</colorMode>
              <scale>1</scale>
              <Icon>%s</Icon> 
              <hotSpot x="0.5"  y="0.5" 
                  xunits="fraction" yunits="fraction"/>    <!-- kml:vec2Type -->
            </IconStyle>
          </Style>
        """ % (os.path.join(const.image_dir, "KML","eventUbapt.png")) )
        self.g.write("""
          <Style id="grampsUbirth"> 
            <BalloonStyle>
              <!-- a background color for the balloon -->
              <color>00ffffff</color>
              <!-- styling of the balloon text -->
              <text><![CDATA[
              <b><font color="#CC0000" size="+3">$[name]</font></b>
              <br/><br/>
              $[description]
              <br/>
              <br/>
              <!-- insert the to/from hyperlinks -->
              $[geDirections]
              ]]></text>
            </BalloonStyle>
            <IconStyle id="grampsIcon">
              <color>ffffffff</color>
              <colorMode>normal</colorMode>
              <scale>1</scale>
              <Icon>%s</Icon> 
              <hotSpot x="0.5"  y="0.5" 
                  xunits="fraction" yunits="fraction"/>    <!-- kml:vec2Type -->
            </IconStyle>
          </Style>
        """ % (os.path.join(const.image_dir, "KML","eventUbirth.png")) )
        self.g.write("""
          <Style id="grampsUdeath"> 
            <BalloonStyle>
              <!-- a background color for the balloon -->
              <color>00ffffff</color>
              <!-- styling of the balloon text -->
              <text><![CDATA[
              <b><font color="#CC0000" size="+3">$[name]</font></b>
              <br/><br/>
              $[description]
              <br/>
              <br/>
              <!-- insert the to/from hyperlinks -->
              $[geDirections]
              ]]></text>
            </BalloonStyle>
            <IconStyle id="grampsIcon">
              <color>ffffffff</color>
              <colorMode>normal</colorMode>
              <scale>1</scale>
              <Icon>%s</Icon> 
              <hotSpot x="0.5"  y="0.5" 
                  xunits="fraction" yunits="fraction"/>    <!-- kml:vec2Type -->
            </IconStyle>
          </Style>
        """ % (os.path.join(const.image_dir, "KML","eventUdeath.png")) )
        self.g.write("""
          <Style id="grampsUburial"> 
            <BalloonStyle>
              <!-- a background color for the balloon -->
              <color>00ffffff</color>
              <!-- styling of the balloon text -->
              <text><![CDATA[
              <b><font color="#CC0000" size="+3">$[name]</font></b>
              <br/><br/>
              $[description]
              <br/>
              <br/>
              <!-- insert the to/from hyperlinks -->
              $[geDirections]
              ]]></text>
            </BalloonStyle>
            <IconStyle id="grampsIcon">
              <color>ffffffff</color>
              <colorMode>normal</colorMode>
              <scale>1</scale>
              <Icon>%s</Icon> 
              <hotSpot x="0.5"  y="0.5" 
                  xunits="fraction" yunits="fraction"/>    <!-- kml:vec2Type -->
            </IconStyle>
          </Style>
        """ % (os.path.join(const.image_dir, "KML","eventUburial.png")) )
        
        # set all placemarks
        for key in self.places:
            place = self.places[key]
            if place[1] and place[2] :
                lat = float(place[1])
                lon = float(place[2])
                lat_offset=0.
                lon_offset=0.
                for event in place[3]:
                    self.g.write("            <Placemark>")
                    self.write_placemark(place[0],lat+lat_offset, \
                        lon+lon_offset, event)
                    self.g.write("            </Placemark>\n")
                    lon_offset += 0.0025
                    if (400 * lon_offset) % 6 == 0 :
                        lat_offset += 0.0025
                        lon_offset = 0.
            else :
                lat=None;lon=None
                for event in place[3]:
                    self.g.write("            <Placemark>")
                    self.write_placemark(place[0],None, None, event)
                    self.g.write("            </Placemark>\n")

        self.g.write("</Document>\n")
        self.g.write("</kml> \n")


    def write_placemark(self,placename, lat, lon, event):
        # the long description: 
        # long name of people to who event is coupled
        # Place: Placename
        # Eventtype: ...... Role: ....
        # Description: ....
        # Date: ....
        description = event[0][0] + '<br />' \
            + _('<b>Place</b>: %(placename)s') %{'placename' : placename} \
            + '<br />' \
            + _('<b>Eventtype</b>: %(eventtype)s')%{'eventtype': event[1]} \
            + ' ' + _('<b>Role</b>: %(role)s')%{'role': event[3]} \
            + '<br />' \
            + _('<b>Description</b>: %(eventdescription)s') % \
                {'eventdescription' : event[2]} \
            + '<br />' \
            + _('<b>Date</b>: %(date)s')%{'date' :event[4]}
        #icon depending on the event, make name
        iconend = ''
        if event[6] == RelLib.EventType.MARRIAGE :
            if event[0][2] == 'Fam' :
                iconend = 'mar'
        elif event[6] == RelLib.EventType.BIRTH :
            if event[0][2] != 'Fam' :
                iconend = 'birth'
        elif event[6] == RelLib.EventType.DEATH :
            if event[0][2] != 'Fam' :
                iconend = 'death'
        elif event[6] == RelLib.EventType.BAPTISM :
            if event[0][2] != 'Fam' :
                iconend = 'bapt'
        elif event[6] == RelLib.EventType.BURIAL :
            if event[0][2] != 'Fam' :
                iconend = 'burial'
        #WATCH out: coordinates are longitude, latitude, height
        try:
            self.g.write("""
            <name>%(name)s</name>
            <visibility>1</visibility>
            <open>1</open>
            <Snippet>%(snip)s</Snippet>
            <description>%(descr)s</description>
            <styleUrl>%(style)s</styleUrl>
            <LookAt>
              <latitude>%(lat).14f</latitude>
              <longitude>%(lon).14f</longitude>
              <range>500</range>
              <tilt>8.3</tilt>
            </LookAt>
            <Point>
              <coordinates>%(lon).14f,%(lat).14f,%(height).14f</coordinates>
            </Point>
""" % { 'name'   : event[0][1], \
        'style'  :'#gramps' + event[0][2]+iconend, \
        'lat' : lat, 'lon' : lon, 'height' : 0, \
        'snip'  : _('Place')+': ' + placename + ';' + _('Event')+': ' + event[2], \
        'descr' : description } )
        except TypeError :
            self.g.write("""
            <name>%(name)s</name>
            <visibility>1</visibility>
            <open>1</open>
            <Snippet>%(snip)s</Snippet>
            <description>%(descr)s</description>
            <styleUrl>%(style)s</styleUrl>
            <LookAt>
              <latitude> </latitude>
              <longitude> </longitude>
              <range>500</range>
              <tilt>8.3</tilt>
            </LookAt>
""" % { 'name'   : event[0][1], \
        'style'  :'#gramps' + event[0][2]+iconend, 
        'snip'  : _('Place')+': ' + placename + ';' + _('Event')+': ' + event[2], \
        'descr' : description } )


#------------------------------------------------------------------------
#
# GNOME/gtk
#
#------------------------------------------------------------------------
import gtk

#-------------------------------------------------------------------------
#
# load GRAMPS libraries
#
#-------------------------------------------------------------------------

import const
from QuestionDialog import ErrorDialog
from PluginUtils import register_report
from ReportBase import CATEGORY_CODE, MODE_GUI, Report, ReportOptions
from ReportBase._ReportDialog import ReportDialog
from NameDisplay import displayer as _nd
from DateHandler import displayer as _dd
#import DateHandler
import PlaceUtils

#------------------------------------------------------------------------
#
# Report class
#
#------------------------------------------------------------------------
class GeographicData:

    def __init__(self,database,person,options_class):
        self.database = database
        self.start_person = person

        options = options_class.handler.options_dict


#------------------------------------------------------------------------
#
# Constant options items
#
#------------------------------------------------------------------------

class _options:
    # internal ID, english option name (for cli), localized option name (for gui)
    formats = (
        ("kml", "Google Earth KML", _("Google Earth KML"), "application/klm"),
    )
    selectopt = (
        ("person", "Current Person", _("Current Person")),
        ("family", "Family of Current Person", _("Family of Current Person")),
        #("filter", "Filtered People", _("Filtered People")),
    )
    

#check if googleearth is installed. 
#  --> problem, googleearth is normally installed locally, not in the PATH variable... TODO
if os.sys.platform == "win32":
    _gge_found = Utils.search_for("googleearth.exe")
else:
    _gge_found = Utils.search_for("googleearth")




#------------------------------------------------------------------------
#
# Options class 
#
#------------------------------------------------------------------------
class GeographicDataOptions(ReportOptions):

    """
    Defines options and provides handling interface.
    """

    def __init__(self,name,person_id=None):
        ReportOptions.__init__(self,name,person_id)

    def set_new_options(self):
        # Options specific for this report
        self.options_dict = {
            'select'     : "person",
            'unknown'   : 0,
            'unknownlatlon' : 0,
            'unknowndate' : 1,
        }
        self.options_help = {
            'select'      : ("=str",
                            "From who to export known geographic data ",
                            [ "%s\t%s" % (item[0],item[1]) for item in _options.selectopt ],
                            False),
            'unknown'    : ("=0/1","Whether to show events with unknown location.",
                            ["Show events with known location","Show events with or without a location"],
                            False),
            'unknownlatlon': ("=0/1","Whether to show events with places with unkown position.",
                            ["Show only places with known latitude/longitude", "Show all places"],
                            False),
            'unknowndate': ("=0/1","Whether to show events with no date specified.",
                            ["Show only events with known date", "Show events with or without date"],
                            False),
        }
        
        
    def make_doc_menu(self,dialog,active=None):
        pass
    
    def add_list(self, options, default):
        "returns combobox of given options and default value"
        box = gtk.ComboBox()
        store = gtk.ListStore(str)
        box.set_model(store)
        cell = gtk.CellRendererText()
        box.pack_start(cell,True)
        box.add_attribute(cell,'text',0)
        index = 0
        for item in options:
            store.append(row=[item[2]])
            if item[0] == default:
                box.set_active(index)
            index = index + 1
        return box

    def add_user_options(self,dialog):
        # Content of the options tab
        #  first combobox with what events to show
        
        self.filter_box = self.add_list(_options.selectopt,
                                        self.options_dict['select'])
        dialog.add_option(_("Filter selection"),
                                self.filter_box,
                                _("Choose people of who to export data."))

        msg = _("Include Events with unknown place")
        self.include_place_unknown = gtk.CheckButton(msg)
        self.include_place_unknown.set_active(self.options_dict['unknown'])
        dialog.add_option(None,
                        self.include_place_unknown,
                        _("Include events with no place/location given."))
                        
        msg = _("Include Events with places with unknown location")
        self.include_placeloc_unknown = gtk.CheckButton(msg)
        self.include_placeloc_unknown.set_active(self.options_dict['unknownlatlon'])
        dialog.add_option(None,
                        self.include_placeloc_unknown,
                        _("Include events even if location (latitude/longitude) of "
                          "the place is unknown."))
                        
        msg = _("Include Events with no date given")
        self.include_date_unknown = gtk.CheckButton(msg)
        self.include_date_unknown.set_active(self.options_dict['unknowndate'])
        dialog.add_option(None,
                        self.include_date_unknown,
                        _("Include events which have no date attached."))

    def parse_user_options(self,dialog):
        self.options_dict['unknown'] = int(self.include_place_unknown.get_active())
        self.options_dict['unknownlatlon'] = \
                int(self.include_placeloc_unknown.get_active())
        self.options_dict['unknowndate'] = \
                int(self.include_date_unknown.get_active())
        self.options_dict['select'] = \
                _options.selectopt[self.filter_box.get_active()][0]



#-------------------------------------------------------------------------
#
#    The Dialog class
#
#-------------------------------------------------------------------------

class GeographicDataDialog(ReportDialog):
    def __init__(self,dbstate,uistate,person):
        self.database = dbstate.db
        self.person = person
        name = "map_view"
        translated_name = _("Map View (Google Earth export)")
        self.options_class = GeographicDataOptions(name)
        self.category = CATEGORY_CODE
        ReportDialog.__init__(self,dbstate,uistate,person,self.options_class,
                              name,translated_name)
        response = self.window.run()
        if response == gtk.RESPONSE_OK:
            try:
                self.make_report()
            except (IOError,OSError),msg:
                ErrorDialog(str(msg))
        elif response == gtk.RESPONSE_DELETE_EVENT:
            return
        self.close()

    def make_doc_menu(self,active=None):
        """Build a one item menu of document types that are
        appropriate for this report."""
        self.format_menu = FormatComboBox()
        self.format_menu.set()

    def make_document(self):
        """Do Nothing.  This document will be created in the
        make_report routine."""
        pass
    
    def setup_style_frame(self):
        """The style frame is not used in this dialog."""
        pass

    def parse_style_frame(self):
        """The style frame is not used in this dialog."""
        pass

    def doc_type_changed(self, obj):
        """The doc type is of no importance in this dialog."""
        pass

    def setup_paper_frame(self):
        """No need for Paper Options in this dialog"""
        pass
    def parse_paper_frame(self):
        """No need for Paper Options in this dialog"""
        pass

    def make_report(self):
        """Create the object that will produce the GraphViz file."""
        GeoGraphicDataExtract(self.database,self.person,self.options_class)


#------------------------------------------------------------------------
#
# Combo Box classes
#
#------------------------------------------------------------------------
class FormatComboBox(gtk.ComboBox):
    """
    Format combo box class.
    
    Trivial class supporting only one format.
    """

    def set(self,tables=0,callback=None,obj=None,active=None):
        self.store = gtk.ListStore(str)
        self.set_model(self.store)
        cell = gtk.CellRendererText()
        self.pack_start(cell,True)
        self.add_attribute(cell,'text',0)
        self.store.append(row=["Google Earth (kml)"])
        self.set_active(0)

    def get_label(self):
        return "Google Earth (kml)"

    def get_reference(self):
        return None

    def get_paper(self):
        return 1

    def get_styles(self):
        return 0

    def get_ext(self):
        return '.kml'

    def get_printable(self):
        _apptype = _options.formats[self.get_active()][3]
        print_label = None
        try:
            mprog = Mime.get_application(_apptype)
    
            if Utils.search_for(mprog[0]):
                print_label = _("Open in %(program_name)s") % { 'program_name':
                                                        mprog[1]}
            else:
                print_label = None
        except:
            print_label = None

        return print_label

    def get_clname(self):
        return 'kml'

class GraphicsFormatComboBox(gtk.ComboBox):
    """
    Format combo box class for graphical (not codegen) report.
    """

    def set(self,active=None):
        self.store = gtk.ListStore(str)
        self.set_model(self.store)
        cell = gtk.CellRendererText()
        self.pack_start(cell,True)
        self.add_attribute(cell,'text',0)
        active_index = 0
        index = 0
        for item in _options.formats:
            self.store.append(row=[item[2]])
            if active == item[0]:
                active_index = index
            index = index + 1
        self.set_active(active_index)

    def get_label(self):
        return _options.formats[self.get_active()][2]

    def get_reference(self):
        return EmptyDoc

    def get_paper(self):
        return 0

    def get_styles(self):
        return 0

    def get_ext(self):
        return '.%s' % _options.formats[self.get_active()][0]

    def get_format_str(self):
        return _options.formats[self.get_active()][0]

    def get_printable(self):
        _apptype = _options.formats[self.get_active()][3]
        print_label = None
        try:
            mprog = Mime.get_application(_apptype)
            if Utils.search_for(mprog[0]):
                print_label = _("Open in %(program_name)s") % { 'program_name':
                                                                mprog[1] }
            else:
                print_label = None
        except:
            print_label = None
        return print_label

    def get_clname(self):
        return 'print'

#------------------------------------------------------------------------
#
# Empty class to keep the BaseDoc-targeted format happy
#
#------------------------------------------------------------------------
class EmptyDoc:
    def __init__(self,styles,type,template,orientation,source=None):
        self.print_req = 0

    def init(self):
        pass
    
    def print_requested(self):
        self.print_req = 1


#------------------------------------------------------------------------
#
#  Class that extracts the geographical data corresponding to the options
#    and sends that to the backend
#
#------------------------------------------------------------------------

class GeoGraphicDataExtract:
  
    def __init__(self,database,person,options_class):
        """
        Creates GeoGraphicDataExtract object that produces data for the report.
        
        The arguments are:

        database        - the GRAMPS database instance
        person          - currently selected person
        options_class   - instance of the Options class for this report

        This report needs the following parameters (class variables)
        that come in the options class.
        
           filter  : from who to export data: current person, his family or 
                        a custom filter
           events  : which events to show
           unknowns: if place has unknown location/event has unknown location, 
                       export it nevertheless
                       
        """
        self.database = database
        self.start_person = person
        
        options = options_class.handler.options_dict
        # selection of who to extract events from 
        
        # selection of which events to show
        self.onlyKnownPlace = not options['unknown']
        self.onlyLatLonPlace = not options['unknownlatlon']
        self.onlyDatePlace = not options['unknowndate']
        self.select = options['select']
        
        self.filename = options_class.get_output()
        
        print 'select is',self.select,', only is',self.onlyKnownPlace, \
                self.onlyLatLonPlace, self.onlyDatePlace,  \
                ', to filename',self.filename
        
        self.obtain_data()
        self.writetobackend()

    def obtain_data(self):
        """we perform the needed lookup in the database. 
          output needed, a dictionary of placemarks, with index the placehandle
            and a placemark being 
        """
        self.placemarks = {}
        
        
        # if family to show, obtain all family handles, then all person's in the family
        #if self.select == 'family':
        #    family_list = self.start_person.get_family_handle_list()
        #    for family_handle in family_list:
        #        family = self.database.get_family_from_handle(family_handle)

        # if person, obtain the events, and place them in the placemarks if needed
        if self.select == 'person' :
            self.obtain_events_person(self.start_person.get_handle(), True)
            self.title = _('Events of %(person)s.' ) %  \
                {'person' : _nd.display(self.start_person)}
            
        if self.select == 'family' :
            start_person_handle = self.start_person.get_handle()
            self.title = _('Events of the family of %(person)s.' ) %  \
                {'person' : _nd.display(self.start_person)}
            person = self.database.get_person_from_handle( start_person_handle)
            self.obtain_events_person(start_person_handle)
            for family_handle in person.get_family_handle_list():
                # output the family events
                self.obtain_events_family(family_handle)
                family = self.database.get_family_from_handle(family_handle)
                # now output the evenfs of all persons in the family
                if family.get_mother_handle() != start_person_handle :
                    self.obtain_events_person(family.get_mother_handle())
                if family.get_father_handle() != start_person_handle :
                    self.obtain_events_person(family.get_father_handle())
                for child_ref in family.get_child_ref_list():
                    child_handle = child_ref.ref
                    self.obtain_events_person(child_handle)

        # filter the data
        #  1. only events with places
        if self.onlyKnownPlace and self.placemarks.has_key('0'):
            del self.placemarks['0']
        for key in self.placemarks.keys() :
            placemark = self.placemarks[key]
            if self.onlyLatLonPlace :
                #  2. only events with places with correct lat/lon
                if not (placemark[1] and placemark[2]) :
                    del self.placemarks[key]
                    continue
                # 3. only events with date given
                if self.onlyDatePlace :
                    nrev = 0
                    for evtdata in placemark[3] :
                        if not (evtdata[4]) : 
                            del placemark[3][nrev]
                        else : 
                            # only if we do not delete need we aument the nr
                            nrev = nrev + 1
            
    def obtain_events_family(self, family_handle) :
        family = self.database.get_family_from_handle(family_handle)
        if not family :
            return
        if not family.get_event_ref_list():
            return
        mother_handle = family.get_mother_handle()
        if mother_handle:
            mother = self.database.get_person_from_handle(mother_handle)
            mother_name = _nd.display(mother)
            mother_name_short = _nd.format_str(mother.get_primary_name(),'%l')\
                + ' ' + _nd.format_str(mother.get_primary_name(),'%f')[0] + '.'
        else:
            mother_name = _("Unknown")
            mother_name_short = mother_name

        father_handle = family.get_father_handle()
        if father_handle:
            father = self.database.get_person_from_handle(father_handle)
            father_name = _nd.display(father)
            father_name_short = _nd.format_str( \
                father.get_primary_name(),'%l') + ' ' +_nd.format_str( \
                father.get_primary_name(),'%f')[0] + '.'
        else:
            father_name = _("Unknown")
            father_name_short = father_name
        name = _('Family %(father_name)s and %(mother_name)s') % { 
            'mother_name' : mother_name,
            'father_name' : father_name }
        short_name = _('Fam. %(father_name_short)s-%(mother_name_short)s') % { 
            'mother_name_short' : mother_name_short,
            'father_name_short' : father_name_short }
        type = 'Fam'
        for event_ref in family.get_event_ref_list():
            self.make_event_placemark((name,short_name,type), event_ref)

        
    def obtain_events_person(self, person_handle, family_events = False) :
        print 'get eve person ' , person_handle
        person = self.database.get_person_from_handle(person_handle)
        if not person :
            return
        name = _nd.display_name(person.get_primary_name())
        short_name = _nd.format_str( \
                person.get_primary_name(),'%l') + ' ' + _nd.format_str( \
                person.get_primary_name(),'%f')[0]+ '.'
        if person.get_gender() == RelLib.Person.FEMALE : 
            type ='F'
        elif person.get_gender() == RelLib.Person.MALE : 
            type ='M'
        else : 
            type ='U'
        event_ref_list = person.get_event_ref_list()
        for event_ref in event_ref_list:
            self.make_event_placemark((name,short_name,type), event_ref)
        # now we add the family events
        if family_events :
            for family_handle in person.get_family_handle_list():
                self.obtain_events_family(family_handle)
                
        
    def make_event_placemark(self, name, event_ref):
            #the event_ref holds the role the person plays in an event
            event = self.database.get_event_from_handle(event_ref.get_reference_handle())
            #date = DateHandler.get_date(event)
            dateobj = event.get_date_object ()
            ph = event.get_place_handle()

            #we have person / eventref / event / place / date, write placemark
            #     data = {person_name = name of person or family
            #             event_name  = name of event (birth, death, ...)
            #             event_descr = description of event
            #             event_role  = role of person in the event
            #             event_date  = date of event
            #             event_dateregular =  is the date regular (one, valid date)
            #            }
            evtName = str( event.get_type() )
            evtDescr= event.get_description()
            evtRole = str(event_ref.get_role())
            evtDate = u''
            evtDateReg = False
            if dateobj:
                if dateobj.is_regular() :
                    evtDateReg = True
                    evtDate = _dd.display(dateobj)
            #if date :
            #    evtDate = date.__str__ #%(date)s % {'date' : date})
            #    evtDateReg = date.is_regular()
            
            placemark = [name, evtName, evtDescr, evtRole, evtDate, \
                        evtDateReg, event.get_type()]
            # we add to dictionary with every place occuring only once
            self.add_placemark(ph, placemark)
            
    def add_placemark(self,placehandle, placemark) :
            # placemark is [title, latitude, longitude, evdata]
            # where evdata is a list of events at this place
            if placehandle:
                place = self.database.get_place_from_handle(placehandle)
                placename = place.get_title()
                lat   = place.get_latitude()
                lon   = place.get_longitude()
                lat, lon = PlaceUtils.conv_lat_lon(lat,lon,"D.D8")
                index = placehandle
            else:
                placename = _('Unknown Place')
                index = '0'
                lat = u''
                lon = u''
            #see if place is present, if not add, if yes, only add data
            if self.placemarks.has_key(index) :
                self.placemarks[index][3].append(placemark)
            else :
                self.placemarks[index]= [placename, lat, lon, [placemark]]
            print self.placemarks
                        
    def writetobackend(self):
         """ write to the actual backend by calling it's writer correctly
         """
         kmlfile = KmlWriter(self.title, self.placemarks)
         kmlfile.write(self.filename)


#------------------------------------------------------------------------
# 
#    Report registration
#
#------------------------------------------------------------------------


register_report(
    name = 'kmlexport',
    category = CATEGORY_CODE,
    report_class = GeographicDataDialog,
    options_class = GeographicDataOptions,
    modes = MODE_GUI ,
    translated_name = _("KML Google Earth Export"),
    status = _("UnStable"),
    description = _("Creates a kml file that can be played/opened in Google Earth."),
    author_name = "Benny Malengier",
    author_email = "bm@cage.UGent.be"
    )
